// KDTree.h: interface for the KDTree class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_KDTREE_H__355F0885_7A06_11D4_B6C2_00A04B0415AD__INCLUDED_)
#define AFX_KDTREE_H__355F0885_7A06_11D4_B6C2_00A04B0415AD__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


#pragma warning( disable : 4786)

#include <list>
#include <map>
#include <utility>
using namespace std;


#include "DblRect.h"

class GraphicObject;


// A KDRegion holds a sorted list of objects.
class KDRegion 
{
  public:
      KDRegion();
      virtual ~KDRegion();
      void AddObject (GraphicObject* obj);
      void RemoveObject (GraphicObject* obj);
	  int Size() {return objects.size();}
      list<GraphicObject*> GetObjects ();
      void Clear () {objects.clear();}
  private: 
	  // bool is just a place holder
      map<GraphicObject*,bool> objects;
};

class KDNode 
{
  public:
      KDNode(DblRect _space, int _threashold, bool _vert);
      virtual ~KDNode();      
	  void Insert(GraphicObject *obj, DblRect rect);
	  void Remove(GraphicObject *obj, DblRect rect);      
	  list<GraphicObject*> *GetObjects (DblRect rect);
  private:    
	  int Size() {return (region) ? region->Size() : left->Size() + right->Size();}
	  void Split();
  private:
	  bool vertSplit;
	  DblRect space;
	  int threashold;
      KDRegion *region;
      KDNode *left;
	  KDNode *right;
};


class KDTree 
{
  public:
      KDTree (DblRect _world, int estNumObj);
      ~KDTree();

      DblRect GetWorldRect () {return worldRect;}
      void InsertObject (GraphicObject* obj, DblRect rect);
      void RemoveObject (GraphicObject* obj);
      list<GraphicObject*>* GetObjects(DblRect rect);
  private: //## implementation    
      DblRect worldRect;
      int levels;
      KDNode *root;
};


#endif // !defined(AFX_KDTREE_H__355F0885_7A06_11D4_B6C2_00A04B0415AD__INCLUDED_)
