#ifndef QuadTree_h
#define QuadTree_h 1

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#pragma warning( disable : 4786)

#include <list>
#include <utility>
using namespace std;


#include "DblRect.h"

class GraphicObject;

class QTRegion 
{
  public:
      QTRegion();
      virtual ~QTRegion();
      void AddObject (GraphicObject* obj);
      void RemoveObject (GraphicObject* obj);
      list<GraphicObject*> GetObjects () {return objects;}
      void Clear () {objects.clear();}
  private: 
      list<GraphicObject*> objects;
};

class QTNode 
{
  public:
	  enum PosEnum {UPPER_LEFT=0, UPPER_RIGHT=1, LOWER_LEFT=2, LOWER_RIGHT=3};
  public:
      QTNode();
      virtual ~QTNode();

      void AddNode (PosEnum position, QTNode* newNode) {if (subNodes[position] == 0) subNodes[position] = newNode;}
      QTNode* GetSubNode (PosEnum position) {return subNodes[position];}
	  QTRegion* GetRegion () {return region;}
      void AddRegion ();
      
  private:    
      QTRegion *region;
      QTNode *subNodes[4];
};


class QuadTree 
{
  public:
      QuadTree (DblRect _world, int _levels=5);
      ~QuadTree();

      DblRect GetWorldRect () {return worldRect;}
      void InsertObject (GraphicObject* obj, DblRect rect);
      void RemoveObject (GraphicObject* obj);
      list<GraphicObject*>* GetObjects(DblRect rect);
      void Clear ();
  private:
      void RegionsFromRect (QTNode* node, DblRect searchRect, DblRect targetRect, list<QTRegion*>* resultList, bool createNew = false, int curLevel = 1);
  private: //## implementation    
      DblRect worldRect;
      int levels;
      QTNode *root;
  private:

	  enum {MASK_ALL_REG = 15, MASK_NOT_REG_03 = 6, MASK_NOT_REG_12 = 9,
		  MASK_NOT_REG_23 = 3, MASK_NOT_REG_01 = 12, MASK_REG_0 = 1,
		  MASK_REG_1 = 2, MASK_REG_2 = 4, MASK_REG_3 = 8};
};


#endif
