// WorkDriver.cpp : Defines the entry point for the console application.
//
#include "stdafx.h"

#include "WorkDriver.h"
#include "GraphicObject.h"
#include "DblRect.h"
#include "QuadTree.h"
#include "KDTree.h"
#include "LinearList.h"

static const DblRect space(0,0,10000,8000);
static const DblRect averageObjectSize(0,0,50,50);
static const DblRect averageScreenSize(0,0,2500,2000);


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// The one and only application object

CWinApp theApp;

using namespace std;


GraphicObject * GetRandomObject(DblRect worldRect)
{
	int width = (((double)rand()) / RAND_MAX) * (worldRect.Width());
	int height = (((double)rand()) / RAND_MAX) * (worldRect.Height());

	// Pick a random spot
	int x = (((double)rand()) / RAND_MAX) * (space.Width() - width);
	int y = (((double)rand()) / RAND_MAX) * (space.Height() - height);

	DblRect bound(x,y,x+width,y+height);
	bound.NormalizeRect();

	return new GraphicObject(bound);
}

DblRect GetRandomScreen(DblRect worldRect)
{
	int width = worldRect.Width();
	int height = worldRect.Height();

	// Pick a random spot
	int x = (((double)rand()) / RAND_MAX) * (space.Width() - width);
	int y = (((double)rand()) / RAND_MAX) * (space.Height() - height);

	DblRect bound(x,y,x+width,y+height);
	bound.NormalizeRect();

	return bound;
}

void InsertionTest(void)
{
	// Set up timers
	LARGE_INTEGER frequency;
	LARGE_INTEGER start;
	LARGE_INTEGER stop;
	double milisec(0);
	QueryPerformanceFrequency(&frequency);

	// Set up random seed
	int seed;
	cout << "Enter random seed: ";
	cin >> seed;
	srand(seed);

	cout << "NumObj\t\tList\tQuad\tK-d" << endl;
	for (int numObjects = 100; numObjects <= 200; numObjects+=10)
	{
		cout << numObjects << "\t\t";
		//////////////////////////////////////////////////////////
		////////////////// LINEAR LIST TESTS	//////////////////
		//////////////////////////////////////////////////////////	
		srand(seed);
		LinearList linearList(space);

		// INSERTION TEST
		QueryPerformanceCounter(&start);
		for (int i = 0; i < numObjects; i++)
		{
			GraphicObject *pNewObject =  GetRandomObject(averageObjectSize);				
			linearList.InsertObject(pNewObject, pNewObject->GetBoundingRect());
		}

		QueryPerformanceCounter(&stop);	
		milisec = ((stop.QuadPart * 1000) / frequency.QuadPart) - ((start.QuadPart * 1000) / frequency.QuadPart);
		cout << milisec << "\t";

		// DELETE 
		list<GraphicObject*> *pList = linearList.GetObjects(space);
		for (std::list<GraphicObject*>::iterator iObj = pList->begin(); iObj != pList->end(); iObj++)
		{
			linearList.RemoveObject(*iObj);
			delete *iObj;
		}
		delete pList;		

		//////////////////////////////////////////////////////////
		////////////////// QUAD TREE TESTS		//////////////////
		//////////////////////////////////////////////////////////
		srand(seed);
		QuadTree quadTree(space);
	
		// INSERTION TEST
		QueryPerformanceCounter(&start);
		// Create and insert <numObjects> into quad tree
		for (i = 0; i < numObjects; i++)
		{
			GraphicObject *pNewObject =  GetRandomObject(averageObjectSize);
			quadTree.InsertObject(pNewObject, pNewObject->GetBoundingRect());
		}

		QueryPerformanceCounter(&stop);	
		milisec = ((stop.QuadPart * 1000) / frequency.QuadPart) - ((start.QuadPart * 1000) / frequency.QuadPart);
		cout <<  milisec << "\t";

		// DELETE TEST
		pList = quadTree.GetObjects(space);
		for (iObj = pList->begin(); iObj != pList->end(); iObj++)
		{
			quadTree.RemoveObject(*iObj);
			delete *iObj;
		}		
		delete pList;

		//////////////////////////////////////////////////////////
		////////////////// K-D TREE TESTS		//////////////////
		//////////////////////////////////////////////////////////
 		srand(seed);	
		KDTree kdTree(space, numObjects);
	
		// INSERTION TEST
		QueryPerformanceCounter(&start);
		// Create and insert <numObjects> into quad tree
		for (i = 0; i < numObjects; i++)
		{
			GraphicObject *pNewObject =  GetRandomObject(averageObjectSize);
			kdTree.InsertObject(pNewObject, pNewObject->GetBoundingRect());
		}

		QueryPerformanceCounter(&stop);	
		milisec = ((stop.QuadPart * 1000) / frequency.QuadPart) - ((start.QuadPart * 1000) / frequency.QuadPart);
		cout << milisec << endl;

		// DELETE 
 		pList = kdTree.GetObjects(space);
		QueryPerformanceCounter(&start);
		for (iObj = pList->begin(); iObj != pList->end(); iObj++)
		{
			kdTree.RemoveObject(*iObj);
			delete *iObj;
		}
		delete pList;
	}
}

void DeleteTest(void)
{
	// Set up timers
	LARGE_INTEGER frequency;
	LARGE_INTEGER start;
	LARGE_INTEGER stop;
	double milisec(0);
	QueryPerformanceFrequency(&frequency);

	// Set up random seed
	int seed;
	cout << "Enter random seed: ";
	cin >> seed;
	srand(seed);

	cout << "NumObj\t\tList\tQuad\tK-d" << endl;
	for (int numObjects = 100; numObjects <= 200; numObjects+=10)
	{
		cout << numObjects << "\t\t";
		//////////////////////////////////////////////////////////
		////////////////// LINEAR LIST TESTS	//////////////////
		//////////////////////////////////////////////////////////	
		srand(seed);
		LinearList linearList(space);

		// INSERTION TEST		
		for (int i = 0; i < numObjects; i++)
		{
			GraphicObject *pNewObject =  GetRandomObject(averageObjectSize);				
			linearList.InsertObject(pNewObject, pNewObject->GetBoundingRect());
		}
	
		// DELETE 
		list<GraphicObject*> *pList = linearList.GetObjects(space);
		QueryPerformanceCounter(&start);
		for (std::list<GraphicObject*>::iterator iObj = pList->begin(); iObj != pList->end(); iObj++)
		{
			linearList.RemoveObject(*iObj);
			delete *iObj;
		}
		QueryPerformanceCounter(&stop);	
		milisec = ((stop.QuadPart * 1000) / frequency.QuadPart) - ((start.QuadPart * 1000) / frequency.QuadPart);
		cout << milisec << "\t";
		delete pList;		

		//////////////////////////////////////////////////////////
		////////////////// QUAD TREE TESTS		//////////////////
		//////////////////////////////////////////////////////////
		srand(seed);
		QuadTree quadTree(space);
	
		// INSERTION TEST		
		// Create and insert <numObjects> into quad tree
		for (i = 0; i < numObjects; i++)
		{
			GraphicObject *pNewObject =  GetRandomObject(averageObjectSize);
			quadTree.InsertObject(pNewObject, pNewObject->GetBoundingRect());
		}

		// DELETE TEST
		pList = quadTree.GetObjects(space);
		QueryPerformanceCounter(&start);
		for (iObj = pList->begin(); iObj != pList->end(); iObj++)
		{
			quadTree.RemoveObject(*iObj);
			delete *iObj;
		}		
		QueryPerformanceCounter(&stop);	
		milisec = ((stop.QuadPart * 1000) / frequency.QuadPart) - ((start.QuadPart * 1000) / frequency.QuadPart);
		cout <<  milisec << "\t";
		delete pList;

		//////////////////////////////////////////////////////////
		////////////////// K-D TREE TESTS		//////////////////
		//////////////////////////////////////////////////////////
 		srand(seed);	
		KDTree kdTree(space, numObjects);
	
		// INSERTION TEST
		
		// Create and insert <numObjects> into quad tree
		for (i = 0; i < numObjects; i++)
		{
			GraphicObject *pNewObject =  GetRandomObject(averageObjectSize);
			kdTree.InsertObject(pNewObject, pNewObject->GetBoundingRect());
		}

		// DELETE 
 		pList = kdTree.GetObjects(space);		
		QueryPerformanceCounter(&start);
		for (iObj = pList->begin(); iObj != pList->end(); iObj++)
		{
			kdTree.RemoveObject(*iObj);
			delete *iObj;
		}
		QueryPerformanceCounter(&stop);	
		milisec = ((stop.QuadPart * 1000) / frequency.QuadPart) - ((start.QuadPart * 1000) / frequency.QuadPart);
		cout << milisec << endl;
		delete pList;
	}
}

void SearchTest(void)
{
	int numSearch = 5;
	// Set up timers
	LARGE_INTEGER frequency;
	LARGE_INTEGER start;
	LARGE_INTEGER stop;
	double milisec(0);
	QueryPerformanceFrequency(&frequency);

	// Set up random seed
	int seed;
	cout << "Enter random seed: ";
	cin >> seed;
	srand(seed);

	cout << "Size\t\tList\tQuad\tK-d" << endl;
	int numObjects=10000;
	srand(seed);
	LinearList linearList(space);

	// INSERTION TEST		
	for (int i = 0; i < numObjects; i++)
	{
		GraphicObject *pNewObject =  GetRandomObject(averageObjectSize);				
		linearList.InsertObject(pNewObject, pNewObject->GetBoundingRect());
	}
		srand(seed);
		QuadTree quadTree(space);

	// INSERTION TEST		
		// Create and insert <numObjects> into quad tree
		for (i = 0; i < numObjects; i++)
		{
			GraphicObject *pNewObject =  GetRandomObject(averageObjectSize);
			quadTree.InsertObject(pNewObject, pNewObject->GetBoundingRect());
		}

 		srand(seed);	
		KDTree kdTree(space, numObjects);
	
		// INSERTION TEST
		
		// Create and insert <numObjects> into quad tree
		for (i = 0; i < numObjects; i++)
		{
			GraphicObject *pNewObject =  GetRandomObject(averageObjectSize);
			kdTree.InsertObject(pNewObject, pNewObject->GetBoundingRect());
		}


	for (int percent = 5; percent <= 100; percent +=5)
	{
		cout << percent << "\t\t";
		//////////////////////////////////////////////////////////
		////////////////// LINEAR LIST TESTS	//////////////////
		//////////////////////////////////////////////////////////	

		// SEARCH TEST		
		DblRect thisSize(0,0, space.Width() * (((double)percent)/((double)100)), space.Height() * (((double)percent)/((double)100)));			
		QueryPerformanceCounter(&start);
		for (i = 0; i < numSearch; i++)
		{
			// Calcualte screen size						
			list<GraphicObject*> *pList = linearList.GetObjects(GetRandomScreen(thisSize));
			delete pList;
		}
		QueryPerformanceCounter(&stop);	
		milisec = ((stop.QuadPart * 1000) / frequency.QuadPart) - ((start.QuadPart * 1000) / frequency.QuadPart);
		cout << milisec << "\t";

		//////////////////////////////////////////////////////////
		////////////////// QUAD TREE TESTS		//////////////////
		//////////////////////////////////////////////////////////
			

		// SEARCH TEST
		QueryPerformanceCounter(&start);
		for (i = 0; i < numSearch; i++)
		{
			GraphicObject *pNewObject =  GetRandomObject(thisSize);				
			list<GraphicObject*> *pList = quadTree.GetObjects(GetRandomScreen(thisSize));
			delete pList;
		}

		QueryPerformanceCounter(&stop);	
		milisec = ((stop.QuadPart * 1000) / frequency.QuadPart) - ((start.QuadPart * 1000) / frequency.QuadPart);
		cout <<  milisec << "\t";


		//////////////////////////////////////////////////////////
		////////////////// K-D TREE TESTS		//////////////////
		//////////////////////////////////////////////////////////

		// SEARCH
		QueryPerformanceCounter(&start);
		for (i = 0; i < numSearch; i++)
		{
			list<GraphicObject*> *pList = kdTree.GetObjects(GetRandomScreen(thisSize));
			delete pList;
		}

		QueryPerformanceCounter(&stop);	
		milisec = ((stop.QuadPart * 1000) / frequency.QuadPart) - ((start.QuadPart * 1000) / frequency.QuadPart);
		cout << milisec << endl;

	}
}

int _tmain(int argc, TCHAR* argv[], TCHAR* envp[])
{
	int nRetCode = 0;

	// initialize MFC and print and error on failure
	if (!AfxWinInit(::GetModuleHandle(NULL), NULL, ::GetCommandLine(), 0))
	{
		// TODO: change error code to suit your needs
		cerr << _T("Fatal Error: MFC initialization failed") << endl;
		nRetCode = 1;
	}
	//InsertionTest();
	//DeleteTest();
	SearchTest();
	cin >> nRetCode;
	return 1;
	//////////////////////////////////////////////////
	////////////////// RANDOM SEED INFO	//////////////
	//////////////////////////////////////////////////
	int seed;
	cout << "Enter random seed: ";
	cin >> seed;
	srand(seed);

	//////////////////////////////////////////////////
	////////////////// TIMER INFO	//////////////////
	//////////////////////////////////////////////////
	LARGE_INTEGER frequency;
	LARGE_INTEGER start;
	LARGE_INTEGER stop;
	double milisec(0);

	// Calculate minimum resolution
	QueryPerformanceFrequency(&frequency);

	//////////////////////////////////////////////////
	////////////////// SETUP		//////////////////
	//////////////////////////////////////////////////
	
	int numObjects(5000);	
	int numSearch(1000);	

	//////////////////////////////////////////////////////////
	////////////////// LINEAR LIST TESTS	//////////////////
	//////////////////////////////////////////////////////////
	cout << "Linear List Test" << endl;
	LinearList linearList(space);

	// INSERTION TEST
	QueryPerformanceCounter(&start);
	for (int i = 0; i < numObjects; i++)
	{
		GraphicObject *pNewObject =  GetRandomObject(averageObjectSize);				
		linearList.InsertObject(pNewObject, pNewObject->GetBoundingRect());
	}

	QueryPerformanceCounter(&stop);	
	milisec = ((stop.QuadPart * 1000) / frequency.QuadPart) - ((start.QuadPart * 1000) / frequency.QuadPart);
	cout << "  Insertion Time: " << milisec << endl;

	// SEARCH TEST
	QueryPerformanceCounter(&start);
	for (i = 0; i < numSearch; i++)
	{
		GraphicObject *pNewObject =  GetRandomObject(averageObjectSize);				
		list<GraphicObject*> *pList = linearList.GetObjects(GetRandomScreen(averageScreenSize));
		delete pList;
	}

	QueryPerformanceCounter(&stop);	
	milisec = ((stop.QuadPart * 1000) / frequency.QuadPart) - ((start.QuadPart * 1000) / frequency.QuadPart);
	cout << "  Search Time: " << milisec << endl;


	// DELETE TEST
	list<GraphicObject*> *pList = linearList.GetObjects(space);
	QueryPerformanceCounter(&start);
	for (std::list<GraphicObject*>::iterator iObj = pList->begin(); iObj != pList->end(); iObj++)
	{
		linearList.RemoveObject(*iObj);
		delete *iObj;
	}
	QueryPerformanceCounter(&stop);	
	milisec = ((stop.QuadPart * 1000) / frequency.QuadPart) - ((start.QuadPart * 1000) / frequency.QuadPart);
	cout << "  Delete Time: " << milisec << endl;
	delete pList;

	//////////////////////////////////////////////////////////
	////////////////// QUAD TREE TESTS		//////////////////
	//////////////////////////////////////////////////////////
	srand(seed);
	cout << "Quad List Test" << endl;
	QuadTree quadTree(space);
	
	// INSERTION TEST
	QueryPerformanceCounter(&start);
	// Create and insert <numObjects> into quad tree
	for (i = 0; i < numObjects; i++)
	{
		GraphicObject *pNewObject =  GetRandomObject(averageObjectSize);
		quadTree.InsertObject(pNewObject, pNewObject->GetBoundingRect());
	}

	QueryPerformanceCounter(&stop);	
	milisec = ((stop.QuadPart * 1000) / frequency.QuadPart) - ((start.QuadPart * 1000) / frequency.QuadPart);
	cout << "  Insertion Time: " << milisec << endl;

	// SEARCH TEST
	QueryPerformanceCounter(&start);
	for (i = 0; i < numSearch; i++)
	{
		GraphicObject *pNewObject =  GetRandomObject(averageObjectSize);				
		list<GraphicObject*> *pList = quadTree.GetObjects(GetRandomScreen(averageScreenSize));
		delete pList;
	}

	QueryPerformanceCounter(&stop);	
	milisec = ((stop.QuadPart * 1000) / frequency.QuadPart) - ((start.QuadPart * 1000) / frequency.QuadPart);
	cout << "  Search Time: " << milisec << endl;

	// DELETE TEST
	pList = quadTree.GetObjects(space);
	QueryPerformanceCounter(&start);
	for (iObj = pList->begin(); iObj != pList->end(); iObj++)
	{
		quadTree.RemoveObject(*iObj);
		delete *iObj;
	}
	QueryPerformanceCounter(&stop);	
	milisec = ((stop.QuadPart * 1000) / frequency.QuadPart) - ((start.QuadPart * 1000) / frequency.QuadPart);
	cout << "  Delete Time: " << milisec << endl;
	delete pList;

	//////////////////////////////////////////////////////////
	////////////////// K-D TREE TESTS		//////////////////
	//////////////////////////////////////////////////////////
//	while (true)
	{
 		srand(seed);

	cout << "k-D Tree Test" << endl;
	KDTree kdTree(space, numObjects);
	
	// INSERTION TEST
	QueryPerformanceCounter(&start);
	// Create and insert <numObjects> into quad tree
	for (i = 0; i < numObjects; i++)
	{
		GraphicObject *pNewObject =  GetRandomObject(averageObjectSize);
		kdTree.InsertObject(pNewObject, pNewObject->GetBoundingRect());
	}

	QueryPerformanceCounter(&stop);	
	milisec = ((stop.QuadPart * 1000) / frequency.QuadPart) - ((start.QuadPart * 1000) / frequency.QuadPart);
	cout << "  Insertion Time: " << milisec << endl;


	// SEARCH TEST
	QueryPerformanceCounter(&start);
	for (i = 0; i < numSearch; i++)
	{
		GraphicObject *pNewObject =  GetRandomObject(averageObjectSize);				
		list<GraphicObject*> *pList = kdTree.GetObjects(GetRandomScreen(averageScreenSize));
		delete pList;
	}

	QueryPerformanceCounter(&stop);	
	milisec = ((stop.QuadPart * 1000) / frequency.QuadPart) - ((start.QuadPart * 1000) / frequency.QuadPart);
	cout << "  Search Time: " << milisec << endl;


	// DELETE TEST
 	pList = kdTree.GetObjects(space);
	QueryPerformanceCounter(&start);
	for (iObj = pList->begin(); iObj != pList->end(); iObj++)
	{
		kdTree.RemoveObject(*iObj);
		delete *iObj;
	}
	QueryPerformanceCounter(&stop);	
	milisec = ((stop.QuadPart * 1000) / frequency.QuadPart) - ((start.QuadPart * 1000) / frequency.QuadPart);
	cout << "  Delete Time: " << milisec << endl;
	delete pList;
	}
	///////////////////
	// Wait for user //
	///////////////////
	cout << "Finished!" << endl;
	char temp;
	cin >> temp;

	return nRetCode;
		
}

